//cl65 -O -Cl -t atari -Wl -D__SYSTEM_CHECK__=1,-D__RESERVED_MEMORY__=1 arpeggiator2.c -o arpeggiator2.com
#include <unistd.h>
#include "pokey.h"

#define PUTS(str) write(STDOUT_FILENO, str, sizeof str - 1)

void wait1(void)
{
    __asm__("lda $14");
    __asm__("wait: cmp $14");
    __asm__("beq wait");
}

#define AMP_LFO_SIZE 20
typedef struct
{
    byte amp[AMP_LFO_SIZE];
} AmpLFO;

#define PITCH_LFO_SIZE 8
#define PITCH_LFO_STEP 5
typedef struct
{
    byte transpose[PITCH_LFO_SIZE];
} PitchLFO;

const AmpLFO amp_lfo1 = {{0,0,0,0,0,6,3,1,0,0,0,0,0,0,0,8,4,2,1,0}};
const AmpLFO amp_lfo2 = {{9,6,5,3,2,7,4,3,3,2,9,5,3,3,2,6,4,3,2,2}};
const AmpLFO amp_lfo3 = {{6,3,1,0,0,0,0,0,0,0,8,4,2,1,0,0,0,0,0,0}};
const AmpLFO amp_lfo4 = {{7,4,3,3,2,9,5,3,3,2,6,4,3,2,2,9,6,5,3,2}};

const PitchLFO pitch_lfo1 = {{0,0,2,5,2,4,0,0}};
const PitchLFO pitch_lfo2 = {{5,7,10,7,9,5,5,5}};

//                 A3   B3  E4  F#4   G4  A4   B4  E5  F#5   G5   A5  B5
const byte f1[] = {68, 147, 43, 182, 128, 29, 198, 18, 214, 184, 139, 94};
const byte f2[] = { 6,   5,  4,   3,   3,  3,   2,  2,   1,   1,   1,  1};

const byte multiply[10 * 16] =
{
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,
    0,0,0,0,1,1,1,1,1,1,1,1,2,2,2,2,
    0,0,0,1,1,1,1,1,2,2,2,2,2,3,3,3,
    0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,
    0,0,1,1,1,2,2,2,3,3,3,4,4,4,5,5,
    0,0,1,1,2,2,2,3,3,4,4,4,5,5,6,6,
    0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,
    0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,
    0,1,1,2,2,3,4,4,5,5,6,7,7,8,8,9,
};

void pattern(const byte root, const byte amp)
{
    byte i = 0, j, k, n, a;
    for (k = 0; k < PITCH_LFO_SIZE; ++k)
        for (j = PITCH_LFO_STEP; j > 0; --j)
        {
            n = pitch_lfo1.transpose[k] + root;
            P1.audf1 = P0.audf1 = f1[n];
            P1.audf2 = P0.audf2 = f2[n];
            n = pitch_lfo2.transpose[k] + root;
            P1.audf3 = P0.audf3 = f1[n];
            P1.audf4 = P0.audf4 = f2[n];

            P1.audc1 = P0.audc1 = 0xc0 | amp_lfo1.amp[i];
            a = amp_lfo2.amp[i] << 4 | amp;
            P1.audc2 = P0.audc2 = 0xc0 | multiply[a];
            P1.audc3 = P0.audc3 = 0xc0 | amp_lfo3.amp[i];
            a = amp_lfo4.amp[i] << 4 | amp;
            P1.audc4 = P0.audc4 = 0xc0 | multiply[a];

            ++i;
            if (i == AMP_LFO_SIZE)
                i = 0;

            wait1();
        }
}

int main(void)
{
    byte i, root;
    PUTS("arpeggiator2/2018-04-15\n");

    P1.skctl = P0.skctl = 3;
    P1.audctl = P0.audctl = 64 + 32 + 16 + 8;
    root = 1;

    for (i = 16; i > 0; --i)
        pattern(root, 0);

    for (i = 0; i < 16; ++i)
        pattern(root, i);

    for (;;)
    {
        root ^= 1;
        for (i = 32; i > 0; --i)
            pattern(root, 15);
    }
}
