//cl65 -O -Cl -t atari -Wl -D__SYSTEM_CHECK__=1,-D__RESERVED_MEMORY__=1 seq2b.c -o seq2b.com
#include <unistd.h>
#include "pokey.h"

#define PUTS(str) write(STDOUT_FILENO, str, sizeof str - 1)

void wait1(void)
{
    __asm__("lda $14");
    __asm__("wait: cmp $14");
    __asm__("beq wait");
}

#define ENV_SIZE 60
typedef struct
{
    byte amp[ENV_SIZE];
    byte loop_begin;
    byte loop_end;
} Envelope;

typedef struct
{
    byte i;
    byte on;
} EnvGenerator;

byte env_next(EnvGenerator* g, const Envelope* env)
{
    const byte a = env->amp[g->i];

    if (g->on)
    {
        if (g->i < env->loop_end)
            ++g->i;
        else
            g->i = env->loop_begin;
    }
    else
    {
        if (g->i != ENV_SIZE - 1)
            ++g->i;
    }

    return a;
}

void note_on(EnvGenerator* g)
{
    g->i = 0;
    g->on = 1;
}

void note_off(EnvGenerator* g)
{
    g->on = 0;
}

#define LFO_SIZE 14
typedef struct
{
    byte amp[LFO_SIZE];
    byte i;
} LFO;

byte lfo_next(LFO* g)
{
    const byte a = g->amp[g->i];
    ++g->i;
    if (g->i == LFO_SIZE)
        g->i = 0;

    return a;
}

const Envelope env1 = {{5,5,5,5,5,4,2,1,1,1,1,1,1,1,4,4,4,3,3,3,2,1,1,1,1,1,1,1,3,3,3,3,3,3,1,1,1,1,1,1,1,1,2,2,2,2,2,2,0,0,0,0,0,0,0,0,1,1,1}, 0, 13};

const Envelope env2 = {{1,1,1,1,1,1,1,1,5,4,4,4,4,2,1,1,1,1,1,1,1,1,4,4,3,3,3,2,1,1,1,1,1,1,1,1,3,2,2,2,2,1,1,1,1,1,1,1,0,0,2,1,1,1,1}, 59, 59};

EnvGenerator gen1 = {ENV_SIZE - 1, 0};
EnvGenerator gen2 = {ENV_SIZE - 1, 0};
LFO lfo = {{7,5,4,3,2,2,1,5,3,2,2,1,1,0}, 0};

#define SCALE1_SIZE 9
//                             C1  Db1  Eb1   F1   G1  Ab1   C2  Db2  Eb2
const byte f2[SCALE1_SIZE] = {239, 226, 201, 179, 159, 150, 119, 112, 100};
const byte f4[SCALE1_SIZE] = {160, 151, 134, 120, 106, 100,  80,  75,  67};

#define D(t) ((t) * 2 * LFO_SIZE)
#define TRACK1_SIZE 16
const byte track1[TRACK1_SIZE] = {3,4,5,2,0,6,1,2,3,4,5,2,0,6,7,8};

const byte note1_duration[TRACK1_SIZE] =
{
    D(4),D(2),D(2),D(3),D(1),D(2),D(1),D(1),D(4),D(2),D(2),D(3),D(1),D(2),D(1),D(1)
};

#define SCALE2_SIZE 6
//                            Eb4   F4   G4  Ab4  Bb4   C5
const byte f1[SCALE1_SIZE] = {181, 161, 143, 135, 119, 106};

#define TRACK2_SIZE 14
const byte track2[TRACK2_SIZE] = {1,0,3,2,5,3,2,1,0,3,4,5,1,2};

const byte note2_duration[TRACK2_SIZE] =
{
    D(4),D(2),D(2),D(4),D(2),D(1),D(1),D(4),D(2),D(2),D(4),D(2),D(1),D(1)
};

int main(void)
{
    byte i1, i2, j1, j2, n;
    PUTS("seq2b/2017-11-11\n");

    P0.skctl = 3;
    P0.audctl = 64 + 32 + 4 + 2 + 1;
    i1 = i2 = j1 = j2 = 0;
    for (;;)
    {
        if (j1 == 0)
        {
            j1 = note1_duration[i1];
            n = track1[i1];
            P0.audf2 = f2[n];
            P0.audf4 = f4[n];
            ++i1;
            if (i1 == TRACK1_SIZE)
                i1 = 0;
        }

        if (j2 == 0)
        {
            j2 = note2_duration[i2];
            n = track2[i2];
            P0.audf3 = P0.audf1 = f1[n];
            STROBE(P0.stimer);
            note_on(&gen1);
            note_on(&gen2);
            ++i2;
            if (i2 == TRACK2_SIZE)
                i2 = 0;
        }

        if (j2 == 40)
            note_off(&gen1);

        P0.audc1 = 0x20 | env_next(&gen1, &env1);
        P0.audc3 = 0x20 | env_next(&gen2, &env2);
        P0.audc2 = 0xa0 | lfo_next(&lfo);
        --j1;
        --j2;
        wait1();
    }
}
